/******************************************************************************/
/* WebCoM 1.0 - Web Course Manager                                            */
/*                                                                            */
/* Copyright (C) 2002 Elaine Quintino da Silva & Dilvan de Abreu Moreira.     */
/*                                                                            */
/* This library is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU Lesser General Public                 */
/* License as published by the Free Software Foundation; either               */
/* version 2.1 of the License, or (at your option) any later version.         */
/*                                                                            */
/* This library is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          */
/* Lesser General Public License for more details.                            */
/*                                                                            */
/* You should have received a copy of the GNU Lesser General Public           */
/* License along with this library; if not, write to the Free Software        */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                            */
/* Contact:                                                                   */
/*       Eletronic mail: elainesilva@zipmail.com or dilvan@computer.org       */
/*       Paper mail: Av. trabalahdor So Carlense, 400 Cx. Postal 668,        */
/*                   So Carlos,So Paulo                                     */
/*                   Brasil CEP 13560-970                                     */
/******************************************************************************/

package course.reviews;

import java.awt.*;
import java.io.*;
import java.sql.*;
import java.util.*;

import agents.View;
import agents.SQL;
import agents.security.Ticket;
import course.util.UtilFunctions;

/** Class for selection of one assignment for allocating of reviewers.*/
public class AssignmentReviewSelection implements View, Serializable {
    Ticket tic;
    String assignmentSelected = null;
    int classSelected = 0;
    Vector listAssignments;
    public static String ASSIGNMENT_ACCESS = "Assignment";
    
    transient AssignmentSelectionInterface assignmentSelectionInterface;
    
    /** Method for setting variables. It sets the selected class.*/
    public void setVariable(String classSelected) {
        StringTokenizer analex = new StringTokenizer(classSelected);
        String token = analex.nextToken();
        try {
            this.classSelected = Integer.parseInt(analex.nextToken());
        } catch (Exception e) {}
    }
    
    
    /** Method for creation of new instance from the View class.*/
    public Object createView(Ticket tic, SQL sql) throws Exception {
        this.tic = tic;
        int countAssignments = 0;
        sql.init(tic.resource);
        // test the user type
        if (!tic.type.equals("administrator") && !tic.type.equals("monitor"))
            throw new RuntimeException("You don't have permissions to access this tool.");
        
        listAssignments = new Vector();
        // Select all of the active classes that has reviews
        ResultSet rs = sql.executeQuery("SELECT id, review_date, use_review FROM assignments WHERE class='" +
        classSelected + "' ORDER BY id");
        
        java.util.Date date = new java.util.Date();
        for (;rs.next();) {
            if (rs.getString(3).equals("true"))
                if (UtilFunctions.verifyDate(date,rs.getDate(2)) >= 0) {
                    listAssignments.addElement(ASSIGNMENT_ACCESS + " " + rs.getInt(1));
                    countAssignments++;
                }
            
        }
        sql.close();
        if (countAssignments == 0) throw new RuntimeException("There is not assignments to make review.");
        return this;
    }
    
    /** Method for creation of a graphic interface that allow to select a class operation.*/
    public Panel initView() {
        Panel principal = new Panel();
        principal.setLayout(new FlowLayout());
        
        assignmentSelectionInterface = new AssignmentSelectionInterface(listAssignments,"Class " + classSelected);
        
        principal.add(assignmentSelectionInterface.showInterface());
        return principal;
    }
    
    /** Method for validation of the graphic interface objects.*/
    public boolean validateView() {
        assignmentSelected = assignmentSelectionInterface.assignmentList.getSelectedItem();
        return true;
    }
    
    /** Method that invokes the class for execution of the operation selected.*/
    public synchronized Object updateView(SQL sql) throws Exception {
        ReviewerView reviewerView = new ReviewerView();
        reviewerView.setVariable(classSelected, assignmentSelected);
        return reviewerView.createView(tic,sql);
    }
}
